          PROGRAM CREATE_ZONE_AVERAGES_SINE
c
c   This program first creates a computational grid or mesh of N
c   zones .The grid begins at 0.0 and has a total length of 1.0.
c   It consists of N+1 lefthand interface locations.
c
c   Zone averages over this grid are then found by finding the 
c   average value of sin(ax) in each zone.  the coefficient a
c   is chosen such that there is an integral number of sinewaves
c   on the interval (0,1).
c
c   A file "OUTPUT.DAT" is created which may be used as input for
c   subsequent example programs.

          ALLOCATABLE ::  XL(:), A_AVG(:)

c
c   The number of grid zones is selected interactively
c
      PRINT *,' Please enter desired number of grid zones (integer):'
      READ *,N
c
c   The number of periods is selected interactively
c
      PRINT *,' Kindly enter desired number of periods (integer):'
      READ *,NWAVES
c
c   The type of grid (uniform, non-uniform, discontinuous) is 
c                                              selected interactively
      PRINT *,' Please enter -1 for a discontinuous grid'
      PRINT *,'               0 for a non-uniform grid or'
      PRINT *,'              +1 for a uniform grid:'
      READ *,IGRID

      PRINT *,' Thank you. '
      PRINT *,'   You have requested ',NWAVES,
     ^        ' sine waves  over',N,' zones.'

          ALLOCATE (XL(N+1), A_AVG(N))

          X_BEGIN = 0.0
          X_LENGTH = 1.0


      IF (IGRID .EQ. 1 ) THEN
          PRINT *,' on a uniform grid'
          CALL CREATE_UNIFORM_GRID( XL(1), X_BEGIN, X_LENGTH, N )
      ELSE 
         IF (IGRID .EQ. 0 ) THEN
             PRINT *,' on a non-uniform grid'
             CALL CREATE_NONUNIFORM_GRID( XL(1), X_BEGIN, X_LENGTH, N )
         ELSE
            IF (IGRID .EQ. -1 ) THEN
                PRINT *,' on a discontinuous-uniform grid'
               CALL CREATE_DISCONTINUOUS_GRID( XL(1), X_BEGIN, 
     ^                                                  X_LENGTH, N )
            ELSE
c
c   unknown grid type
               PRINT *,' UNKNOWN GRID TYPE:',IGRID
               CALL ABORT()
            ENDIF
         ENDIF
      ENDIF


c
c   The functional form is sin(alpha*x)
c
c   The average of this function on the interval (x1,x2) is
c
c              -[cos(alpha*x2) - cos(alpha*x1)]/[alpha*(x2-x1)]
c
          ALPHA = 6.28318530717958 * FLOAT(NWAVES)
          DO I = 1,N
             XLEFT = COS( ALPHA*XL(I) )
             XRIGHT = COS( ALPHA*XL(I+1) )
             DX = XL(I+1)-XL(I)

             A_AVG(I) = - (XRIGHT - XLEFT)/(ALPHA*DX)
          ENDDO
          OPEN (UNIT=11, FILE='OUTPUT.AVG')
	  write(11,50) N
50    FORMAT(1X,I10)

          WRITE (6,100)
100   FORMAT(6X,'I',6X,'XMID',10X,'A_AVG',/,40('_'))

          DO I=1,N
            XMID = 0.5 * (XL(I+1)+XL(I))
            WRITE (6,200) I, XMID, A_AVG(I)
200     FORMAT (4X,I4,t10,1P,2(E12.5,3x))
            WRITE (11,300)  XL(I), A_AVG(I)
300     FORMAT (4X,1P,2(E12.5,3x))
          ENDDO
            WRITE (11,300)  XL(N+1), A_AVG(N)
            CLOSE(11)

          DEALLOCATE (XL, A_AVG)

          END ! PROGRAM CREATE_ZONE_AVERAGES_SINE

